"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Etl = exports.EtlState = void 0;
const rxjs_1 = require("rxjs");
const MapTransformer_1 = require("./transformers/MapTransformer");
var EtlState;
(function (EtlState) {
    EtlState[EtlState["Running"] = 0] = "Running";
    EtlState[EtlState["Stopped"] = 1] = "Stopped";
    EtlState[EtlState["Error"] = 2] = "Error";
})(EtlState = exports.EtlState || (exports.EtlState = {}));
/**
 * ETL Class. Instantiate one and add as many extractors, transformers and loaders as you want.
 * Then start the whole process with ".start()".
 *
 * This processor is modular, you can find other implemented loaders and extractors in the README
 */
class Etl {
    constructor(context) {
        this._extractors = [];
        this._generalTransformers = [];
        this._transformers = [];
        this._loaders = [];
        this._state = EtlState.Stopped;
        this._context = null;
        this.setContext(context);
    }
    get extractors() {
        return this._extractors;
    }
    get generalTransformers() {
        return this._generalTransformers;
    }
    get transformers() {
        return this._transformers;
    }
    get loaders() {
        return this._loaders;
    }
    get state() {
        return this._state;
    }
    setContext(context) {
        if (this._state !== EtlState.Stopped) {
            this._state = EtlState.Error;
            throw new Error('Tried to set context on invalid state.');
        }
        this._context = context;
        return this;
    }
    addExtractor(extract) {
        this._extractors.push(extract);
        return this;
    }
    addGeneralTransformer(transformer) {
        this._generalTransformers.push(transformer);
        return this;
    }
    addTransformer(transformer) {
        this.addGeneralTransformer(new MapTransformer_1.MapTransformer(transformer));
        this._transformers.push(transformer);
        return this;
    }
    addLoader(loader) {
        this._loaders.push(loader);
        return this;
    }
    /**
     * Starts the etl process. First, all extractors are run in parallel and deliver their results into an observable.
     * Once the buffer gets a result, it transfers all objects through the transformers (one by one).
     * After that, the transformed results are run through all loaders in parallel.
     *
     * @returns {Observable<any>} Observable that completes when the process is finished,
     *                            during the "next" process step you get update on how many are processed yet.
     *                            Throws when any step produces an error.
     */
    start(observable = rxjs_1.EMPTY) {
        this._state = EtlState.Running;
        const o = (0, rxjs_1.merge)(observable, ...this._extractors.map((extractor) => extractor.read(this._context)));
        return this._generalTransformers
            .reduce((observable, transformer) => transformer.process(observable, this._context), o)
            .pipe((0, rxjs_1.mergeMap)((object) => (0, rxjs_1.merge)(...this._loaders.map((loader) => loader.write(object, this._context)))))
            .pipe((0, rxjs_1.tap)({
            error: (err) => {
                this._state = EtlState.Error;
                return (0, rxjs_1.throwError)(() => err);
            },
            complete: () => {
                this._state = EtlState.Stopped;
            },
        }));
    }
    /**
     * Resets the whole Etl object. Deletes all modifiers and resets the state.
     */
    reset() {
        this._extractors = [];
        this._transformers = [];
        this._loaders = [];
        this._state = EtlState.Stopped;
        this._context = null;
    }
}
exports.Etl = Etl;
